@@ -1,5 +1,5 @@
 #
-# $Id: 59cb78bc6d79cb4801cea37e379c9ee58fae03b1 $
+# $Id: 733f7084ff1dca50994cba55c8a05aba8d82eb31 $
 #
 # Copyright (c) 1999-2006 Minero Aoki
 #
@@ -503,7 +503,7 @@ module Racc
         set.update tmp
         return set
       end
-      tok = h = nil
+      tok = nil
       set.update_a t.heads
       t.heads.each do |ptr|
         tok = ptr.dereference
